import subprocess
import os
import click
from colorama import Fore, Style


class Executor:
    def __init__(self):
        self.initial_dir = os.getcwd()
        self.current_dir = self.initial_dir

    def execute_shell_command(self, command):
        click.echo(
            f"{Fore.YELLOW}Executing shell command: {command}{Style.RESET_ALL}")

        # Handle cd command separately
        if command.strip().startswith('cd '):
            new_dir = command.split(maxsplit=1)[1]
            new_dir = os.path.join(self.current_dir, new_dir)
            if os.path.isdir(new_dir):
                self.current_dir = new_dir
                click.echo(
                    f"{Fore.GREEN}Changed directory to: {self.current_dir}{Style.RESET_ALL}")
                return ""
            else:
                raise Exception(f"Directory not found: {new_dir}")

        # Set environment variable to force non-interactive mode for npm
        env = os.environ.copy()
        env['CI'] = 'true'

        try:
            process = subprocess.Popen(
                command,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                env=env,
                cwd=self.current_dir
            )

            # Read output in real-time
            while True:
                output = process.stdout.readline()
                if output == '' and process.poll() is not None:
                    break
                if output:
                    click.echo(output.strip())

            return_code = process.poll()

            if return_code != 0:
                error_output = process.stderr.read()
                raise Exception(
                    f"Command failed with return code {return_code}. Error: {error_output}")

            return process.stdout.read()
        except Exception as e:
            raise Exception(f"Error executing command: {e}")

    def perform_file_operation(self, operation, filename, content=None, force=False):
        # Always use absolute path based on the initial directory
        full_path = os.path.join(self.initial_dir, filename)

        click.echo(
            f"{Fore.YELLOW}Performing {operation} operation on file: {full_path}{Style.RESET_ALL}")

        if operation.lower() == 'create':
            os.makedirs(os.path.dirname(full_path), exist_ok=True)
            with open(full_path, 'w') as f:
                f.write(content)
            click.echo(
                f"{Fore.GREEN}File created successfully.{Style.RESET_ALL}")
            return True
        elif operation.lower() == 'update':
            if os.path.exists(full_path):
                with open(full_path, 'r') as f:
                    existing_content = f.read()
                click.echo(
                    f"{Fore.CYAN}Existing content:{Style.RESET_ALL}\n{existing_content}\n")
                click.echo(
                    f"{Fore.CYAN}New content:{Style.RESET_ALL}\n{content}\n")
                if force or click.confirm("Do you want to replace the entire file content?"):
                    with open(full_path, 'w') as f:
                        f.write(content)
                    click.echo(
                        f"{Fore.GREEN}File updated successfully.{Style.RESET_ALL}")
                    return True
                else:
                    click.echo(
                        f"{Fore.YELLOW}File update skipped.{Style.RESET_ALL}")
                    return False
            else:
                click.echo(
                    f"{Fore.YELLOW}File does not exist. Creating new file.{Style.RESET_ALL}")
                os.makedirs(os.path.dirname(full_path), exist_ok=True)
                with open(full_path, 'w') as f:
                    f.write(content)
                click.echo(
                    f"{Fore.GREEN}File created successfully.{Style.RESET_ALL}")
                return True
        elif operation.lower() == 'delete':
            if os.path.exists(full_path):
                if force or click.confirm(f"Are you sure you want to delete {full_path}?"):
                    os.remove(full_path)
                    click.echo(
                        f"{Fore.GREEN}File deleted successfully.{Style.RESET_ALL}")
                    return True
                else:
                    click.echo(
                        f"{Fore.YELLOW}File deletion skipped.{Style.RESET_ALL}")
                    return False
            else:
                click.echo(
                    f"{Fore.YELLOW}File not found: {full_path}. Deletion skipped.{Style.RESET_ALL}")
                return False
        else:
            raise ValueError(f"Unknown file operation: {operation}")



import json
import os

METADATA_FILE = 'project_metadata.json'


def get_project_context():
    if os.path.exists(METADATA_FILE):
        with open(METADATA_FILE, 'r') as f:
            metadata = json.load(f)
        return json.dumps(metadata, indent=2)
    return "{}"



def get_instruction_prompt():
    return """
# Instructions for Claude: Project Setup Assistant

You are a project setup assistant. Generate steps in the proper order, with prerequisite steps first to avoid errors. 
Use the current directory for all operations, including creating new projects like Next.js, Rails, or Python apps.

Your responses should follow this JSON format:

{
  "explanation": "A brief explanation of the steps, if necessary",
  "steps": [
    {
      "type": "shell",
      "command": "command to execute"
    },
    {
      "type": "file",
      "operation": "CREATE",
      "filename": "path/to/file.ext",
      "content": "file content here"
    },
    {
      "type": "file",
      "operation": "UPDATE",
      "filename": "path/to/existing/file.ext",
      "content": "content to append or replace"
    },
    {
      "type": "file",
      "operation": "DELETE",
      "filename": "path/to/file/to/delete.ext"
    }
  ]
}

Important guidelines:
1. Always use the current directory for project creation. For example:
   - Use `npx create-next-app@latest . --typescript --eslint --tailwind --src-dir --app --import-alias "@/*" --use-npm` instead of creating a new subdirectory.
2. Include non-interactive flags for commands that might prompt for user input.
3. Use relative paths for all file operations.
4. Do not use 'cd' commands. All operations should be relative to the current directory.
5. If a command fails due to existing files, provide alternative steps to handle the situation (e.g., suggesting file removal or using a different directory).
6. Strictly generate json only, no other extra words.

Ensure all steps are executable and maintain a logical flow of operations.
"""



import click
import os
from dotenv import load_dotenv
from .claude_api import call_claude_api, generate_description
from .claude_parser import parse_claude_response, pretty_print_commands
from .executor import Executor
from .project_metadata import ProjectMetadataManager
from colorama import init, Fore, Style

# Initialize colorama
init(autoreset=True)

# Load environment variables
load_dotenv()


def print_step(step_number, total_steps, message):
    click.echo(
        f"{Fore.CYAN}[{step_number}/{total_steps}] {message}{Style.RESET_ALL}")


def print_success(message):
    click.echo(f"{Fore.GREEN}✔ {message}{Style.RESET_ALL}")


def print_error(message):
    click.echo(f"{Fore.RED}✘ {message}{Style.RESET_ALL}")


def print_info(message):
    click.echo(f"{Fore.YELLOW}ℹ {message}{Style.RESET_ALL}")


def get_file_content(filename):
    if os.path.exists(filename):
        with open(filename, 'r') as f:
            return f.read()
    return None


def create_drd_json_if_needed(directory):
    drd_json_path = os.path.join(directory, 'drd.json')
    if not os.path.exists(drd_json_path):
        if click.confirm("No drd.json found. Do you want to create one?"):
            metadata_manager = ProjectMetadataManager(directory)
            metadata_manager.save_metadata({
                "project_name": os.path.basename(directory),
                "last_updated": "",
                "files": []
            })
            print_success(f"Created drd.json in {directory}")
            return metadata_manager
    return ProjectMetadataManager(directory)


def apply_fix_commands(fix_commands, executor):
    all_outputs = []
    total_steps = len(fix_commands)

    for i, fix_cmd in enumerate(fix_commands, 1):
        print(fix_cmd, "----")
        if fix_cmd['type'] == 'explanation':
            print_info(
                f"Step {i}/{total_steps}: Explanation: {fix_cmd['content']}")
            all_outputs.append(
                f"Step {i}/{total_steps}: Explanation - {fix_cmd['content']}")
        elif fix_cmd['type'] == 'shell':
            print_info(
                f"Step {i}/{total_steps}: Running the fix: {fix_cmd['command']}")
            try:
                output = executor.execute_shell_command(fix_cmd['command'])
                print_success(
                    f"Step {i}/{total_steps}: Successfully executed: {fix_cmd['command']}")
                if output:
                    click.echo(
                        f"{Fore.MAGENTA}Command output:{Style.RESET_ALL}\n{output}")
                all_outputs.append(
                    f"Step {i}/{total_steps}: Shell command - {fix_cmd['command']}\nOutput: {output}")
            except Exception as e:
                error_message = f"Step {i}/{total_steps}: Error executing fix command: {fix_cmd['command']}\nError details: {str(e)}"
                print_error(error_message)
                all_outputs.append(error_message)
                return False, i, str(e), "\n".join(all_outputs)
        elif fix_cmd['type'] == 'file':
            print_info(
                f"Step {i}/{total_steps}: Updating file: {fix_cmd['filename']}")
            try:
                operation_performed = executor.perform_file_operation(
                    'UPDATE',
                    fix_cmd['filename'],
                    fix_cmd.get('content'),
                    force=True
                )
                if operation_performed:
                    print_success(
                        f"Step {i}/{total_steps}: Successfully updated file: {fix_cmd['filename']}")
                    all_outputs.append(
                        f"Step {i}/{total_steps}: File update - {fix_cmd['filename']} - Success")
                else:
                    error_message = f"Step {i}/{total_steps}: Failed to update file: {fix_cmd['filename']}"
                    print_error(error_message)
                    all_outputs.append(error_message)
                    return False, i, "File operation failed", "\n".join(all_outputs)
            except Exception as e:
                error_message = f"Step {i}/{total_steps}: Error updating file: {fix_cmd['filename']}\nError details: {str(e)}"
                print_error(error_message)
                all_outputs.append(error_message)
                return False, i, str(e), "\n".join(all_outputs)
    return True, total_steps, None, "\n".join(all_outputs)


def handle_error_with_claude(error, cmd, executor, depth=0, previous_context=""):
    if depth > 3:  # Limit recursion depth
        print_error(
            "Max error handling depth reached. Unable to resolve the issue.")
        return False

    print_error(f"Error executing command: {error}")
    error_trace = traceback.format_exc()

    error_query = get_error_analysis_prompt(
        cmd, error, error_trace, previous_context)

    print_info("Sending error information to Claude for analysis...")
    response = call_claude_api(error_query)
    fix_commands = parse_claude_response(response)

    print_info("Claude's suggested fix:")
    print_info("Applying Claude's suggested fix...")
    print(fix_commands, "----")
    fix_applied, step_completed, error_message, all_outputs = apply_fix_commands(
        fix_commands, executor)

    if fix_applied:
        print_success("All fix steps successfully applied.")
        print_info("Fix application details:")
        click.echo(all_outputs)
        return True
    else:
        print_error(f"Failed to apply the fix at step {step_completed}.")
        print_error(f"Error message: {error_message}")
        print_info("Fix application details:")
        click.echo(all_outputs)

        # Prepare a new error query with the failed fix attempt information
        new_error_query = get_error_analysis_prompt(
            {"type": "fix", "command": f"apply fix step {step_completed}"},
            Exception(error_message),
            all_outputs,
            previous_context + "\n\n" + error_query
        )

        # Recursively try to fix the error in applying the fix
        return handle_error_with_claude(Exception(new_error_query),
                                        {"type": "fix",
                                            "command": f"apply fix step {step_completed}"},
                                        executor, depth + 1, new_error_query)


def create_drd_json(directory):
    metadata_manager = ProjectMetadataManager(directory)
    metadata_manager.save_metadata({
        "project_name": os.path.basename(directory),
        "last_updated": "",
        "files": []
    })
    print_success(f"Created drd.json in {directory}")


@click.command()
@click.option('--query', prompt='Your query for Claude', help='The query to send to Claude API')
@click.option('--debug', is_flag=True, help='Print debug information')
def claude_cli(query, debug):
    print_info("Starting Claude CLI tool...")

    current_dir = os.getcwd()
    executor = Executor()
    metadata_manager = ProjectMetadataManager(current_dir)

    print_info("Sending query to Claude API...")
    project_context = metadata_manager.get_project_context()
    response = call_claude_api(
        f"{project_context}\n\nUser query: {query}", include_context=True)
    if debug:
        print_info("Raw response from Claude API:")
        print(response)
    print_success("Received response from Claude API.")

    commands = parse_claude_response(response)
    if not commands:
        print_error(
            "Failed to parse Claude's response. No commands to execute.")
        return

    if debug:
        print_info("Parsed commands:")
        pretty_print_commands(commands)
    print_info(f"Parsed {len(commands)} commands from Claude's response.")

    for i, cmd in enumerate(commands):
        if cmd['type'] == 'explanation':
            print_info(f"Explanation: {cmd['content']}")
            continue

        print_step(i+1, len(commands), f"Processing {cmd['type']} command...")

        max_retries = 3
        for attempt in range(max_retries):
            try:
                if cmd['type'] == 'shell':
                    output = executor.execute_shell_command(cmd['command'])
                    print_success(f"Executed shell command: {cmd['command']}")
                    if output:
                        click.echo(
                            f"{Fore.MAGENTA}Command output:{Style.RESET_ALL}\n{output}")
                elif cmd['type'] == 'file':
                    operation_performed = executor.perform_file_operation(
                        cmd['operation'],
                        cmd['filename'],
                        cmd.get('content'),
                        force=True
                    )

                    if operation_performed:
                        description = generate_description(
                            cmd['filename'], cmd.get('content', ''))
                        metadata_manager.update_file_metadata(
                            cmd['filename'],
                            cmd['filename'].split('.')[-1],
                            cmd.get('content', ''),
                            description
                        )
                        print_success(
                            f"Performed {cmd['operation']} operation on file: {cmd['filename']}")
                        print_info(f"Generated description: {description}")
                    else:
                        print_info(
                            f"File operation skipped for: {cmd['filename']}")
                break  # If successful, break the retry loop
            except Exception as e:
                print_info(
                    f"Error occurred (Attempt {attempt+1}/{max_retries}). Attempting to fix with Claude's assistance.")
                if handle_error_with_claude(e, cmd, executor, metadata_manager):
                    print_info(
                        "Fix applied successfully. Retrying the original command.")
                else:
                    print_error(
                        f"Unable to fix the error after attempt {attempt+1}.")
                    if attempt == max_retries - 1:
                        print_info(
                            "Max retries reached. Skipping this command.")
                        break
        else:
            print_error(
                f"Failed to execute command after {max_retries} attempts. Skipping and moving to the next command.")

    print_success("Claude CLI tool execution completed.")


def handle_error_with_claude(error, cmd, executor, metadata_manager, depth=0, previous_context=""):
    if depth > 3:  # Limit recursion depth
        print_error(
            "Max error handling depth reached. Unable to resolve the issue.")
        return False

    print_error(f"Error executing command: {error}")
    error_trace = traceback.format_exc()

    project_context = metadata_manager.get_project_context()
    error_query = f"""
    Project context:
    {project_context}

    Previous context:
    {previous_context}

    An error occurred while executing the following command:
    {cmd['type']}: {cmd.get('command') or cmd.get('filename')}
    
    Error details:
    {error_trace}
    
    Please analyze this error and provide a JSON response with the following structure:
    {{
        "explanation": "Brief explanation of the error and proposed fix",
        "steps": [
            {{
                "type": "shell" or "file",
                "command" or "filename": "command to execute or file to modify",
                "content": "file content if type is file"
            }}
        ]
    }}
    Ensure the steps are executable and will resolve the issue. Strictly only respond with json, no extra words before or after.
    """

    print_info("Sending error information to Claude for analysis...")
    response = call_claude_api(error_query, include_context=True)
    fix_commands = parse_claude_response(response)

    print_info("Claude's suggested fix:")
    print_info("Applying Claude's suggested fix...")
    fix_applied, step_completed, error_message, all_outputs = apply_fix_commands(
        fix_commands, executor, metadata_manager)

    if fix_applied:
        print_success("All fix steps successfully applied.")
        print_info("Fix application details:")
        click.echo(all_outputs)
        return True
    else:
        print_error(f"Failed to apply the fix at step {step_completed}.")
        print_error(f"Error message: {error_message}")
        print_info("Fix application details:")
        click.echo(all_outputs)

        # Recursively try to fix the error in applying the fix
        return handle_error_with_claude(Exception(error_message),
                                        {"type": "fix",
                                            "command": f"apply fix step {step_completed}"},
                                        executor, metadata_manager, depth + 1, all_outputs)


def apply_fix_commands(fix_commands, executor, metadata_manager):
    # ... [keep the existing implementation, but update file metadata after each successful file operation]
    # Example:
    if cmd['type'] == 'file' and operation_performed:
        description = generate_description(
            cmd['filename'], cmd.get('content', ''))
        metadata_manager.update_file_metadata(
            cmd['filename'],
            cmd['filename'].split('.')[-1],
            cmd.get('content', ''),
            description
        )
    # ...


if __name__ == '__main__':
    claude_cli()



def get_error_analysis_prompt(cmd, error, error_trace, previous_context):
    return f"""
Previous context:
{previous_context}

An error occurred while executing the following command:
{cmd['type']}: {cmd.get('command') or cmd.get('filename')}

Error details:
{error_trace}

Please analyze this error and provide a JSON response with the following structure:
{{
    "explanation": "Brief explanation of the error and proposed fix",
    "steps": [
        {{
            "type": "shell" or "file",
            "command" or "filename": "command to execute or file to modify",
            "content": "file content if type is file"
        }}
    ]
}}
Ensure the steps are executable and will resolve the issue. Strictly only respond with json, no extra words before or after.
"""



import json
import os
from datetime import datetime


class ProjectMetadataManager:
    def __init__(self, project_dir):
        self.project_dir = project_dir
        self.metadata_file = os.path.join(self.project_dir, 'drd.json')
        self.metadata = self.load_metadata()

    def load_metadata(self):
        if os.path.exists(self.metadata_file):
            with open(self.metadata_file, 'r') as f:
                return json.load(f)
        return {"project_name": os.path.basename(self.project_dir), "last_updated": "", "files": []}

    def save_metadata(self):
        with open(self.metadata_file, 'w') as f:
            json.dump(self.metadata, f, indent=2)

    def update_file_metadata(self, filename, file_type, content, description=None):
        self.metadata['last_updated'] = datetime.now().isoformat()

        # Create or update file entry
        file_entry = next(
            (f for f in self.metadata['files'] if f['filename'] == filename), None)
        if file_entry is None:
            file_entry = {'filename': filename}
            self.metadata['files'].append(file_entry)

        file_entry.update({
            'type': file_type,
            'last_modified': datetime.now().isoformat(),
            'content_preview': content[:100] + ('...' if len(content) > 100 else ''),
            'description': description or file_entry.get('description', '')
        })

        self.save_metadata()

    def get_file_metadata(self, filename):
        return next((f for f in self.metadata['files'] if f['filename'] == filename), None)

    def get_project_context(self):
        return json.dumps(self.metadata, indent=2)



import json
import os
import requests
from .utils import get_project_context
from .prompts.claude_instructions import get_instruction_prompt
from .prompts.file_operations import get_file_identification_prompt, get_file_description_prompt


def call_claude_api(query, include_context=False):
    api_key = os.getenv('CLAUDE_API_KEY')
    if not api_key:
        raise ValueError("CLAUDE_API_KEY not found in environment variables")

    headers = {
        'x-api-key': f'{api_key}',
        'Content-Type': 'application/json',
        'Anthropic-Version': '2023-06-01'
    }

    instruction_prompt = get_instruction_prompt()

    if include_context:
        project_context = get_project_context()
        query = f"Project context:\n{project_context}\n\nUser query: {query}"

    data = {
        'model': 'claude-3-5-sonnet-20240620',
        'system': instruction_prompt,
        'messages': [
            {'role': 'user', 'content': query}
        ],
        'max_tokens': 2000
    }

    response = requests.post(
        'https://api.anthropic.com/v1/messages', json=data, headers=headers)
    response.raise_for_status()

    resp = response.json()['content'][0]['text']
    print(resp, "resp")
    return resp


def identify_file(query):
    api_key = os.getenv('CLAUDE_API_KEY')
    if not api_key:
        raise ValueError("CLAUDE_API_KEY not found in environment variables")

    headers = {
        'x-api-key': f'{api_key}',
        'Anthropic-Version': '2023-06-01',
        'Content-Type': 'application/json'
    }

    project_context = get_project_context()
    system_message = get_file_identification_prompt()

    data = {
        'model': 'claude-3-5-sonnet-20240620',
        'system': system_message,
        'messages': [
            {'role': 'user', 'content': f"Project context:\n{project_context}\n\nUser query: {query}"}
        ],
        'max_tokens': 1000
    }

    response = requests.post(
        'https://api.anthropic.com/v1/messages', json=data, headers=headers)
    response.raise_for_status()

    try:
        resp = response.json()['content'][0]['text']
        print(resp, "---")
        file_info = json.loads(response.json()['content'][0]['text'])
        return file_info['filename'], file_info['exists']
    except json.JSONDecodeError:
        raise ValueError(
            "Failed to parse file identification response from Claude")


def generate_description(filename, content):
    api_key = os.getenv('CLAUDE_API_KEY')
    if not api_key:
        raise ValueError("CLAUDE_API_KEY not found in environment variables")

    headers = {
        'x-api-key': f'{api_key}',
        'Content-Type': 'application/json',
        'Anthropic-Version': '2023-06-01'
    }

    system_message = get_file_description_prompt()

    data = {
        'model': 'claude-3-5-sonnet-20240620',
        'system': system_message,
        'messages': [
            {'role': 'user',
                'content': f"Filename: {filename}\n\nContent:\n{content[:1000]}..."}
        ],
        'max_tokens': 1000
    }

    response = requests.post(
        'https://api.anthropic.com/v1/messages', json=data, headers=headers)
    response.raise_for_status()

    description = response.json()['content'][0]['text'].strip()
    return description[:100]  # Ensure the description is not too long



import json
import re


def parse_claude_response(response):
    try:
        # First, try to parse the response as-is
        parsed_response = json.loads(response)
        return process_parsed_response(parsed_response)
    except json.JSONDecodeError:
        # If parsing fails, attempt to fix common issues
        fixed_response = fix_json_response(response)
        try:
            parsed_response = json.loads(fixed_response)
            return process_parsed_response(parsed_response)
        except json.JSONDecodeError as e:
            print(f"Error parsing JSON response: {e}")
            return []


def fix_json_response(response):
    # Remove any text before the first '{' and after the last '}'
    response = re.sub(r'^[^{]*', '', response)
    response = re.sub(r'[^}]*$', '', response)

    # Escape newlines and other control characters in string values
    def escape_string(match):
        return json.dumps(match.group(0))[1:-1]  # Remove outer quotes

    response = re.sub(r'"[^"\\]*(?:\\.[^"\\]*)*"', escape_string, response)

    return response


def process_parsed_response(parsed_response):
    commands = parsed_response.get('steps', [])
    if 'explanation' in parsed_response:
        commands.insert(0, {
            'type': 'explanation',
            'content': parsed_response['explanation']
        })
    return commands


def pretty_print_commands(commands):
    for i, cmd in enumerate(commands, 1):
        print(f"Command {i}:")
        for key, value in cmd.items():
            if key == 'content' and len(value) > 100:
                print(f"  {key}: {value[:100]}... (truncated)")
            else:
                print(f"  {key}: {value}")
        print()
